package com.moreirahelder.busdashboard;

import android.content.Context;
import android.preference.PreferenceManager;
import android.widget.Toast;

import org.eclipse.paho.android.service.MqttAndroidClient;
import org.eclipse.paho.client.mqttv3.IMqttActionListener;
import org.eclipse.paho.client.mqttv3.IMqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.IMqttToken;
import org.eclipse.paho.client.mqttv3.MqttCallback;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * Created by helder on 13-05-2015.
 */
public class BusMQTT implements MqttCallback {

    private Context context;

    private Thread trip;
    volatile boolean stopTrip;
    volatile boolean finish;

    private int npassengers = 0;

    private MqttAndroidClient client;
    private Locations locations;
    private MainCallbacks callback;

    public BusMQTT(Context context, MainCallbacks callback) {
        this.context = context;
        this.callback = callback;
        this.locations = Locations.getInstance(context);
        callback.setStatusDisconnected("Disconnected");
    }

    public void start() {
        MemoryPersistence persistence = new MemoryPersistence();
        client = new MqttAndroidClient(context, "tcp://" +
                PreferenceManager.getDefaultSharedPreferences(context).getString("address", "192.168.160.71") +
                ":" + PreferenceManager.getDefaultSharedPreferences(context).getString("port", "1884"),
                "BusDashboard",
                persistence);
        client.setCallback(this);
        try {
            client.connect(context, new IMqttActionListener() {

                @Override
                public void onSuccess(IMqttToken iMqttToken) {
                    try {
                        client.subscribe(new String[]{"./DB/BUS/Entrance", "./DB/BUS/Exit"}, new int[]{1, 1});
                    } catch (MqttException e) {
                        e.printStackTrace();
                    }
                    callback.setStartEnabled(false);
                    npassengers = 0;
                    callback.setNPassengers(0);
                    stopTrip = false;
                    callback.setStatusConnected("Connected");
                    trip = new Thread(new Runnable() {
                        public void run() {
                            finish = false;
                            sendMessage("start");
                            while (!stopTrip) {
                                try {
                                    Thread.sleep(2000);
                                } catch (InterruptedException e) {
                                    e.printStackTrace();
                                }
                                sendMessage("moving");
                            }
                            sendMessage("finish");
                            try {
                                client.disconnect();
                                callback.setStatusDisconnected("Disconnected");
                            } catch (MqttException e) {
                                e.printStackTrace();
                            }
                            finish = true;
                            callback.setStartEnabled(true);
                        }
                    });
                    trip.start();
                }

                @Override
                public void onFailure(IMqttToken iMqttToken, Throwable throwable) {
                    Toast.makeText(context, "Could not connect", Toast.LENGTH_LONG).show();
                    callback.setStatusDisconnected("Disconnected");
                    client.unregisterResources();
                }
            });
        } catch (MqttException e) {
            client.unregisterResources();
            e.printStackTrace();
            Toast.makeText(context, "Exception on connect", Toast.LENGTH_LONG).show();
        }
    }

    public void stop() {
        stopTrip = true;
    }

    public boolean sendMessage(String status) {
        try {
            JSONObject obj = new JSONObject();
            obj.put("status", status);
            obj.put("location", locations.getCurrentLocation());
            obj.put("speed", callback.getSpeed());
            if (status.equals("moving")) {
                obj.put("n_passengers", npassengers);
            }
            if(status.equals("start")){
                obj.put("routeid", callback.getRouteID());
                obj.put("driverid", callback.getDriverID());
            }
            MqttMessage message = new MqttMessage();
            message.setPayload(obj.toString().getBytes());
            client.publish("./DC/BUS/Status", message);
            return true;
        } catch (MqttException ex) {
            return false;
        } catch (JSONException e) {
            e.printStackTrace();
            return false;
        }
    }

    @Override
    public void connectionLost(Throwable throwable) {
        callback.setStatusDisconnected("Disconnected");
        callback.setStartEnabled(true);
    }

    @Override
    public void messageArrived(String topic, MqttMessage message) throws Exception {
        JSONObject obj = new JSONObject(message.toString());
        String status = obj.getString("status");
        switch (topic) {
            case "./DB/BUS/Entrance":
                if (status.equals("ok")) {
                    callback.showValidEntry();
                    callback.setNPassengers(++npassengers);
                } else {
                    callback.showInvalidEntry(obj.getString("info"));
                }
                break;
            case "./DB/BUS/Exit":
                if (status.equals("ok")) {
                    callback.showValidExit();
                    if (npassengers > 0)
                        callback.setNPassengers(--npassengers);
                } else {
                    callback.showInvalidExit(obj.getString("info"));
                }
                break;
        }

    }

    @Override
    public void deliveryComplete(IMqttDeliveryToken iMqttDeliveryToken) {

    }

    public void exit(){
        while(!finish){
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        client.unregisterResources();
    }
}
