package com.moreirahelder.busdashboard;

import android.app.Activity;
import android.app.Dialog;
import android.content.Intent;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.os.Handler;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;


public class MainActivity extends Activity implements MainCallbacks {

    private BusMQTT bus;
    private TextView npassengers;

    private Button start_button;
    private Button stop_button;

    private TextView entry;
    private TextView entry_error;
    private TextView status;
    private TextView speed_text;

    private String routeid;
    private String driverid;
    private int speed;

    Runnable entry_reset;
    Handler entry_handler;

    MediaPlayer mvalid;
    MediaPlayer minvalid;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        npassengers = (TextView) this.findViewById(R.id.npassengers);
        entry = (TextView) this.findViewById(R.id.entry);
        entry_error = (TextView) this.findViewById(R.id.entryError);
        status = (TextView) this.findViewById(R.id.status);

        resetEntry();
        setNPassengers(0);

        bus = new BusMQTT(this, this);

        start_button = (Button) this.findViewById(R.id.buttonStart);
        start_button.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                bus.start();
            }
        });

        stop_button = (Button) this.findViewById(R.id.buttonStop);
        stop_button.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                bus.stop();
            }
        });

        entry_handler = new Handler();
        entry_reset = new Runnable() {
            @Override
            public void run() {
                resetEntry();
            }
        };

        mvalid = MediaPlayer.create(this, R.raw.valid);
        minvalid = MediaPlayer.create(this, R.raw.invalid);

        speed = 0;
        speed_text = (TextView) this.findViewById(R.id.speed);
        updateSpeed();
        Button up_button = (Button) this.findViewById(R.id.upbutton);
        Button down_button = (Button) this.findViewById(R.id.downbutton);

        up_button.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {
                if (speed < 180)
                    speed++;
                updateSpeed();
                return true;
            }
        });

        down_button.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {
                if (speed > 0)
                    speed--;
                updateSpeed();
                return true;
            }
        });

        final Dialog dialog = new Dialog(this);
        dialog.setContentView(R.layout.driver_dialog);
        dialog.setTitle("Driver Settings");
        dialog.setCancelable(false);
        ((TextView) dialog.findViewById(android.R.id.title)).setGravity(Gravity.CENTER);

        Button dialogButton = (Button) dialog.findViewById(R.id.buttonConfirm);
        dialogButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                EditText edriverid = ((EditText) dialog.findViewById(R.id.driverid));
                EditText erouteid = ((EditText) dialog.findViewById(R.id.routeid));
                if (edriverid.getText().toString().equals("") || erouteid.getText().toString().equals(""))
                    return;
                routeid = erouteid.getText().toString();
                driverid = edriverid.getText().toString();
                dialog.dismiss();
            }
        });

        dialog.show();
    }

    public void updateSpeed() {
        speed_text.setText("" + speed);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();
        if (id == R.id.action_settings) {
            startActivity(new Intent(this, Settings.class));
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    @Override
    public void setNPassengers(int nPassengers) {
        npassengers.setText("" + nPassengers);
    }

    @Override
    public void showValidEntry() {
        resetEntry();
        entry.setText("Valid Entry");
        entry.setBackgroundColor(getResources().getColor(android.R.color.holo_green_dark));

        mvalid.start();
        entry_handler.removeCallbacks(entry_reset);
        entry_handler.postDelayed(entry_reset, 5000);
    }

    @Override
    public void showInvalidEntry(String error) {
        resetEntry();
        entry.setText("Invalid Entry");
        entry.setBackgroundColor(getResources().getColor(android.R.color.holo_red_dark));
        entry_error.setText(error);
        entry_error.setVisibility(View.VISIBLE);

        minvalid.start();
        entry_handler.removeCallbacks(entry_reset);
        entry_handler.postDelayed(entry_reset, 5000);
    }

    @Override
    public void showValidExit() {
        resetEntry();
        entry.setText("Valid Exit");
        entry.setBackgroundColor(getResources().getColor(android.R.color.holo_green_dark));

        mvalid.start();
        entry_handler.removeCallbacks(entry_reset);
        entry_handler.postDelayed(entry_reset, 1000);
    }

    @Override
    public void showInvalidExit(String error) {
        resetEntry();
        entry.setText("Invalid Exit");
        entry.setBackgroundColor(getResources().getColor(android.R.color.holo_red_dark));
        entry_error.setText(error);
        entry_error.setVisibility(View.VISIBLE);

        minvalid.start();
        entry_handler.removeCallbacks(entry_reset);
        entry_handler.postDelayed(entry_reset, 5000);
    }

    @Override
    public void setStartEnabled(final boolean flag) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                start_button.setEnabled(flag);
            }
        });
    }

    @Override
    public String getDriverID() {
        return driverid;
    }

    @Override
    public String getRouteID() {
        return routeid;
    }

    @Override
    public int getSpeed() {
        return speed;
    }

    @Override
    public void setStatusConnected(final String stat) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                status.setTextColor(getResources().getColor(android.R.color.holo_green_dark));
                status.setText(stat);
            }
        });
    }

    @Override
    public void setStatusDisconnected(final String stat) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                status.setTextColor(getResources().getColor(android.R.color.holo_red_dark));
                status.setText(stat);
            }
        });
    }

    public void resetEntry() {
        entry.setBackgroundColor(getResources().getColor(android.R.color.transparent));
        entry.setText("");
        entry_error.setText("");
        entry_error.setVisibility(View.INVISIBLE);
    }

    @Override
    protected void onDestroy(){
        bus.stop();
        bus.exit();
        super.onDestroy();
    }
}
